'''
External libraries
==================

Kivy comes with other python/C libraries:

* :mod:`~kivy.lib.ddsfile` - used for parsing and saving
  `DDS <https://en.wikipedia.org/wiki/DirectDraw_Surface>`_ files.
* :mod:`~kivy.lib.osc` - a modified/optimized version of PyOSC for using
  the `Open Sound Control <https://en.wikipedia.org/wiki/Open_Sound_Control>`_
  protocol.
* :mod:`~kivy.lib.mtdev` - provides support for the
  `Kernel multi-touch transformation library <https://launchpad.net/mtdev>`_.

.. warning::

    Even though Kivy comes with these external libraries, we do not provide any
    support for them and they might change in the future.
    Don't rely on them in your code.

'''
